"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ReachedToAttempts_exception_1 = __importDefault(require("./exceptions/ReachedToAttempts.exception"));
const NotFoundKey_exception_1 = __importDefault(require("./exceptions/NotFoundKey.exception"));
const node_cache_1 = __importDefault(require("node-cache"));
const crypto_1 = __importDefault(require("crypto"));
class OneTimeActivationCode {
    constructor(options) {
        this.encodeCode = true;
        this.expiresAfter = 180;
        this.attemptsChance = 0;
        this.cacheSystem = new node_cache_1.default();
        this.timeConversion = (duration) => {
            const portions = [];
            const msInHour = 1000 * 60 * 60;
            const hours = Math.trunc(duration / msInHour);
            if (hours > 0) {
                portions.push(hours + 'h');
                duration = duration - hours * msInHour;
            }
            const msInMinute = 1000 * 60;
            const minutes = Math.trunc(duration / msInMinute);
            if (minutes > 0) {
                portions.push(minutes + 'm');
                duration = duration - minutes * msInMinute;
            }
            const seconds = Math.trunc(duration / 1000);
            if (seconds > 0) {
                portions.push(seconds + 's');
            }
            return portions.join(' ');
        };
        if (options && options.encodeCode !== undefined) {
            this.encodeCode = options.encodeCode;
        }
        if (options && options.expiresAfter) {
            this.expiresAfter = options.expiresAfter;
        }
        if (options && options.attemptsChance) {
            this.attemptsChance = options.attemptsChance;
        }
    }
    set(key, code) {
        const encodedCode = this.encodeCode ? crypto_1.default.createHash('sha256').update(code).digest('hex') : code;
        const activationCode = {
            attempts: 0,
            code: encodedCode,
        };
        this.cacheSystem.set(`otac_${key}`, activationCode, this.expiresAfter);
    }
    get(key) {
        const activationCode = this.cacheSystem.get(`otac_${key}`);
        if (!activationCode) {
            throw new NotFoundKey_exception_1.default();
        }
        return activationCode;
    }
    isValid(key, code) {
        const encodedCode = this.encodeCode ? crypto_1.default.createHash('sha256').update(code).digest('hex') : code;
        const activationCode = this.get(key);
        this.handleAttempts(key, activationCode);
        if (activationCode.code !== encodedCode) {
            return false;
        }
        this.cacheSystem.del(`otac_${key}`);
        return true;
    }
    handleAttempts(key, activationCode) {
        const ttl = (this.cacheSystem.getTtl(`otac_${key}`) || new Date().getTime()) - new Date().getTime();
        if (this.attemptsChance > 0) {
            if (activationCode.attempts >= this.attemptsChance) {
                throw new ReachedToAttempts_exception_1.default(`Sorry, you\'ve reached to ${activationCode.attempts} attempts. Please try again ${this.timeConversion(ttl)} later`);
            }
        }
        activationCode.attempts++;
        this.cacheSystem.set(`otac_${key}`, activationCode, Math.floor(ttl / 1000));
    }
}
exports.default = OneTimeActivationCode;
//# sourceMappingURL=OneTimeActivationCode.js.map